#ifndef CQTHREADHELPER_H
#define CQTHREADHELPER_H

#include <QThread>
#include <stdio.h>
#include <sys/types.h>

#ifndef IN
#define IN
#endif

#ifndef INFOLOG
#define INFOLOG(...) fprintf(stderr, __VA_ARGS__);fprintf(stderr, "\n");
#endif

#ifndef ERRORLOG
#define ERRORLOG INFOLOG
#endif

typedef void (*PFN_THREAD_PROC)(void* pContext);

// UI Thread의 Signal & Slot binding
// cqthreadhelper는 caller의 CQThreadHelper instance를 넘긴다.
// 상속된 QWidget 내부에서 사용한다. (macro에 this를 사용하는데, 그것이 ui로 가정했기 때문)
// ON_MESSAGE_POST : PostMessage(...)처럼 메시지 큐에 넣고 caller(즉, emit)는 바로 리턴된다.
// ON_MESSAGE_SEND : SendMessage(...)처럼 메시지 큐에 넣고 caller(즉, emit)는 해당 slot의 처리 완료까지 대기한다.
#define ON_MESSAGE_POST(cqthreadhelper,signal,slot) cqthreadhelper.connect(this,signal,this,slot,Qt::AutoConnection);
#define ON_MESSAGE_SEND(cqthreadhelper,signal,slot) cqthreadhelper.connect(this,signal,this,slot,Qt::BlockingQueuedConnection);

class CQThreadHelper : public QThread
{
    Q_OBJECT
public:
    explicit CQThreadHelper(QObject *parent = 0);

    ////////////////////////////////////////////
    // 외부 caller는 아래 static 함수로 접근하기 요망
public:
    // Thread Life Time 관련
    bool BeginThread(IN PFN_THREAD_PROC pfnThreadProc, IN void* pContext);
    void WaitThread(void);

protected:
    // 환경 변수
    bool        m_bRunning;
    pthread_t   m_nCallingTID;
    pthread_t   m_nThreadTID;

protected:
    // 실행 변수
    PFN_THREAD_PROC m_pfnThreadProc;
    void*           m_pContext;
    
protected:
    void run();
};

#endif // CQTHREADHELPER_H
